package cn.com.qmth.examcloud.core.examwork.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.SqlUtil;
import cn.com.qmth.examcloud.core.examwork.dao.ExamCourseRelationRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamCourseRelationEntity;
import cn.com.qmth.examcloud.core.examwork.service.ExamCourseService;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamCourseRelationInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.ExamCourseRelationQuery;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ExamCourseServiceImpl implements ExamCourseService {

    private static final Logger log = LoggerFactory.getLogger(ExamCourseServiceImpl.class);

    @Autowired
    private ExamCourseRelationRepo examCourseRelationRepo;

    @Override
    public Page<ExamCourseRelationInfo> getExamCourseRelationListByPage(ExamCourseRelationQuery req) {
        if (req.getPageNo() == null || req.getPageNo() < 1) {
            req.setPageNo(1);
        }

        if (req.getPageSize() == null || req.getPageSize() < 1) {
            req.setPageSize(10);
        }
        if (req.getExamId() == null) {
            throw new StatusException("请先选择考试！");
        }

        Specification<ExamCourseRelationEntity> spec = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("examId"), req.getExamId()));

            if (StringUtils.isNotBlank(req.getCourseCode())) {
                predicates.add(cb.like(root.get("courseCode"), SqlUtil.toSqlRightLike(req.getCourseCode())));
            }
            if (StringUtils.isNotBlank(req.getCourseName())) {
                predicates.add(cb.like(root.get("courseName"), SqlUtil.toSqlRightLike(req.getCourseName())));
            }

            if (req.getWeixinAnswerEnabled() != null) {
                predicates.add(cb.equal(root.get("weixinAnswerEnabled"), req.getWeixinAnswerEnabled()));
            }

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Sort sort = Sort.by(Sort.Order.desc("updateTime"));
        Pageable pageable = PageRequest.of(req.getPageNo() - 1, req.getPageSize(), sort);
        Page<ExamCourseRelationEntity> page = examCourseRelationRepo.findAll(spec, pageable);

        if (!page.hasContent()) {
            return new PageImpl<>(new ArrayList<>(), pageable, 0);
        }

        List<ExamCourseRelationInfo> list = page.getContent().stream()
                .map(entity -> toExamCourseRelationInfo(entity))
                .collect(Collectors.toList());

        return new PageImpl<>(list, pageable, page.getTotalElements());
    }

    @Override
    public List<ExamCourseRelationInfo> getExamCourseRelationList(Long examId, Boolean weixinAnswerEnabled) {
        Specification<ExamCourseRelationEntity> spec = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.equal(root.get("examId"), examId));

            if (weixinAnswerEnabled != null) {
                predicates.add(cb.equal(root.get("weixinAnswerEnabled"), weixinAnswerEnabled));
            }

            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        List<ExamCourseRelationEntity> list = examCourseRelationRepo.findAll(spec);
        return list.stream().map(entity -> toExamCourseRelationInfo(entity)).collect(Collectors.toList());
    }

    @Override
    public List<Long> getExamCourseIds(Long examId) {
        List<Long> courseIds = examCourseRelationRepo.findExamCourseIds(examId);
        if (courseIds == null) {
            return new ArrayList<>();
        }
        return courseIds;
    }

    private ExamCourseRelationInfo toExamCourseRelationInfo(ExamCourseRelationEntity entity) {
        ExamCourseRelationInfo info = new ExamCourseRelationInfo();
        info.setExamId(entity.getExamId());
        info.setCourseId(entity.getCourseId());
        info.setCourseCode(entity.getCourseCode());
        info.setCourseName(entity.getCourseName());
        info.setCourseLevel(entity.getCourseLevel());
        info.setCourseEnable(entity.getCourseEnable());
        info.setWeixinAnswerEnabled(entity.getWeixinAnswerEnabled());
        info.setPassScoreLine(entity.getPassScoreLine());
        info.setGoodScoreLine(entity.getGoodScoreLine());
        return info;
    }

    @Override
    public void batchUpdateWeixinAnswerEnabled(Long examId, Long[] courseIds, boolean enabled) {
        if (examId == null || ArrayUtils.isEmpty(courseIds)) {
            return;
        }

        int effect = examCourseRelationRepo.batchUpdateWeixinAnswerEnabled(examId, courseIds, enabled, new Date());
        log.warn("batchUpdateWeixinAnswerEnabled examId={} courseIds={} enabled={} effectCount={}", examId, courseIds, enabled, effect);
    }

    @Override
    public void updateWeixinAnswerEnabled(Long examId, String courseCode, boolean enabled) {
        if (examId == null || StringUtils.isBlank(courseCode)) {
            return;
        }

        int effect = examCourseRelationRepo.updateWeixinAnswerEnabled(examId, courseCode, enabled, new Date());
        log.warn("updateWeixinAnswerEnabled examId={} courseCode={} enabled={} effectCount={}", examId, courseCode, enabled, effect);
    }

    @Override
    public void settingScoreLine(Long examId, Long courseId, Integer passScoreLine, Integer goodScoreLine) {
        if (examId == null || courseId == null) {
            return;
        }
        int effect = examCourseRelationRepo.updateScoreLine(examId, courseId, passScoreLine, goodScoreLine, new Date());
        log.warn("updateScoreLine examId:{} courseId:{} passScoreLine:{} goodScoreLine:{} effectCount={}",
                examId, courseId, passScoreLine, goodScoreLine, effect);
    }

}
